import numpy as np
import requests
import time
from io import StringIO
from monteprediction import MONTE_URL


def send_in_chunks(df, email, num_chunks, name=None, max_retries=3):
    if name is None:
        name = email[:20]
        print(f'Suggest you supply name argument for the leaderboard. Using {name} for now')
    chunks = np.array_split(df, num_chunks)
    for chunk_no, chunk_df in enumerate(chunks):
        for attempt in range(max_retries):
            try:
                # Metadata and URL setup
                metadata = {'email': email, 'name':name, 'chunk': chunk_no, 'num_chunks': num_chunks}

                # Convert DataFrame chunk to CSV string
                csv_string = chunk_df.to_csv(index=False)

                # Stream the CSV string to the server
                with StringIO(csv_string) as f:
                    response = requests.post(MONTE_URL, params=metadata, data=f)

                # Check response
                if response.ok:
                    print(f"Chunk {chunk_no} of {num_chunks} sent successfully.")
                    break  # Break the retry loop if successful
                else:
                    print(f"Failed to send chunk {chunk_no}, attempt {attempt + 1}. Response: {response.content}")

            except Exception as e:
                print(f"An error occurred: {e}")

            # Optional: wait before retrying
            time.sleep(1)  # Wait for 1 second before retrying

        else:
            print(f"Failed to send chunk {chunk_no} after {max_retries} attempts.")



from monteprediction import KERNEL_RATE
import numpy as np


def compute_score(samples, z, a=KERNEL_RATE):
    distances = np.linalg.norm(samples - z, axis=1)
    return np.sum(np.exp(-a * distances))


from monteprediction.calendarutil import get_last_wednesday, back_to_weekday
import pytest
from datetime import datetime, timedelta, date


def test_get_last_wednesday_day():
    assert get_last_wednesday().weekday() == 2  # 2 represents Wednesday


def test_get_last_wednesday_not_future():
    assert get_last_wednesday() <= datetime.now().date()


def test_back_to_weekday_type():
    assert isinstance(get_last_wednesday(), date)


def test_back_to_weekday_weekday():
    for day in range(0, 5):  # Monday to Friday
        random_day = datetime.now().date() - timedelta(days=(datetime.now().weekday() - day))
        assert back_to_weekday(random_day) == random_day


def test_back_to_weekday_saturday():
    saturday = datetime.now().date() + timedelta(days=(5 - datetime.now().weekday()))
    assert back_to_weekday(saturday) == saturday - timedelta(days=1)


def test_back_to_weekday_sunday():
    sunday = datetime.now().date() + timedelta(days=(6 - datetime.now().weekday()))
    assert back_to_weekday(sunday) == sunday - timedelta(days=2)



if __name__ == "__main__":
    pytest.main(["-v"])



from datetime import datetime, timedelta


def get_last_wednesday():
    today = datetime.now()
    offset = (today.weekday() - 2) % 7
    last_wednesday = today - timedelta(days=offset)
    return last_wednesday.date()


def back_to_weekday(d):
    if d.weekday() == 5:  # Saturday
        end_date = d - timedelta(days=1)  # Previous day (Friday)
    elif d.weekday() == 6:  # Sunday
        end_date = d - timedelta(days=2)  # Two days before (Friday)
    else:
        end_date = d
    return end_date


from monteprediction.calendarutil import back_to_weekday
from datetime import timedelta, datetime
from monteprediction import SPDR_ETFS
import yfinance as yf


def get_most_recent_truth():
    end_date = back_to_weekday(datetime.now() - timedelta(days=1))
    start_date = end_date - timedelta(weeks=3)
    recent_data = yf.download(SPDR_ETFS, start=start_date.date(), end=end_date.date(), interval="1wk")
    recent_weekly_prices = recent_data['Adj Close']
    return recent_weekly_prices.pct_change().dropna().iloc[-1].values
